#ifndef WRAPPER_H
#define WRAPPER_H

// C++
#include <cstdint>
#include <functional>
#include <mutex>
#include <string>
#include <tuple>
#include <variant>
#include <vector>

// LEPCLIB
#if defined(_WIN32)
// Windows
#include <windows.h>
#include <LEPCLIB_RT.h>
using LE_SYSTIME = SYSTEMTIME;
#else
// Linux
#include <LEPC.h>
using ULONG = uint32_t;
#endif

namespace wrapper
{
  /**
   * @brief Throw error for FAIL
   *
   */
  class LEPCError : public std::exception
  {
  public:
    explicit LEPCError(const char *m) : message{m} {}
    const char *what() const noexcept override { return message.c_str(); }

  private:
    std::string message{};
  };

  //! Type definition of monitor data of LE_MONDATA_FORMAT_1
  using LE_MONDATAFORMAT1_FRAMES = std::vector<LE_MONDATAFORMAT1>;
  //! Type definition of monitor data of LE_MONDATA_FORMAT_2
  using LE_MONDATAFORMAT2_FRAMES = std::vector<LE_MONDATAFORMAT2>;
  //! Type definition of monitor data of either LE_MONDATAFORMAT1_FRAMES or LE_MONDATAFORMAT2_FRAMES
  using LE_MONDATAFORMAT_VARIANT = std::variant<LE_MONDATAFORMAT1_FRAMES, LE_MONDATAFORMAT2_FRAMES>;
  //! Type definition of function called in monitor result callback
  using LE_CALLBACK_MONITOR = std::function<void(LE_MONDATAFORMAT_VARIANT &)>;
  //! Type definition of function called in progress callback
  using LE_CALLBACK_PROGRESS = std::function<void(unsigned long, unsigned long)>;

  /* --------------------------------------------------------------------------------
   * Public (Wrapper) functions
   * -------------------------------------------------------------------------------- */
  short getDllVersion();
  bool connect(std::string com);
  bool disconnect();
  LE_SYSTIME getDateTime();
  bool setDateTime(LE_SYSTIME time);
  bool startOnlineCallBack(LE_CALLBACK_MONITOR &callback, int iFormat);
  bool startManualCallBack(LE_CALLBACK_MONITOR &callback, int iFormat, unsigned long ulDelayTime, unsigned long ulIdleTime, char fRepeat);
  bool startBufferCallBack(LE_CALLBACK_MONITOR &callback, int iFormat, unsigned long ulDelayTime, unsigned long ulIdleTime, char fRepeat, LE_BUFSIMDATA eData);
  bool startProgramCallBack(LE_CALLBACK_MONITOR &callback, int iFormat, LE_PROGRAM eProgram);
  bool startProgramCallBack2(LE_CALLBACK_MONITOR &callback, int iFormat, LE_PROGRAM eProgram, LE_CALLBACK_PROGRESS &callback_progress);
  bool startFlowSendCharCallBack(LE_CALLBACK_MONITOR &callback, int iFormat, char fInitial, char cXon, char cXoff, LE_STDTABLENO eTable, unsigned long ulIdleTime);
  bool startFlowSendLineCallBack(LE_CALLBACK_MONITOR &callback, int iFormat, char fInitial, LE_FLOWWATCH eWatch, LE_FLOWOPERATE eOperate, LE_STDTABLENO eTable, unsigned long ulIdleTime);
  bool startFlowRecvCharCallBack(LE_CALLBACK_MONITOR &callback, int iFormat, char fInitial, char cXon, char cXoff, unsigned long ulOnCount, unsigned long ulOffTime);
  bool startFlowRecvLineCallBack(LE_CALLBACK_MONITOR &callback, int iFormat, char fInitial, LE_FLOWWATCH eWatch, LE_FLOWOPERATE eOperate, unsigned long ulOnCount, unsigned long ulOffTime);
  bool startEchoBufferCallBack(LE_CALLBACK_MONITOR &callback, int iFormat, unsigned long ulResponseTime);
  bool startEchoCharCallBack(LE_CALLBACK_MONITOR &callback, int iFormat);
  bool startEchoLoopbackCallBack(LE_CALLBACK_MONITOR &callback, int iFormat, LE_LOOPBACKLINE eLoopbackLine);
  bool startBertContinue(LE_BERTPATTERN ePattern, int flow);
  bool startBertRbit(LE_BERTPATTERN ePattern, LE_BERTCOUNT eCount, int flow);
  bool startBertRuntime(LE_BERTPATTERN ePattern, ULONG ulTestTime, int flow);
  bool startBertRepeat(LE_BERTPATTERN ePattern, short sTimeResolution, int flow);
  bool restartCallBack(LE_CALLBACK_MONITOR &callback, int iFormat);
  bool startOnline();
  bool stop();
  int getRunStatus();
  bool emulationKey(int iCode);
  LE_LTABLE getTable(int number);
  bool setTable(int number, LE_LTABLE table);
  bool setTableString(int number, std::string str);
  LE_LPROGRAM getProgram(int number);
  bool setProgram(int number, LE_LPROGRAM program);
  bool setTrigger(int number, LE_TRIGGERFACTOR factor, LE_TRIGGERACTION action);
  bool setTriggerOff(int number);
  std::tuple<char, LE_TRIGGERFACTOR, LE_TRIGGERACTION> getTrigger(int number);
  LE_LPROTOCOL getProtocol();
  int setProtocol(LE_LPROTOCOL eProtocol);
  LE_SETUP_PROTOCOL getProtocolSetup();
  bool setProtocolSetup(LE_SETUP_PROTOCOL protocol);

#ifdef __linux__
  ULONG getTimer(int number);
  ULONG getCounter(int number);
#endif

  bool interfaceRS232C(LE_IFMODE eMode, LE_IFPOLARITY ePolarity);
  bool interfaceRS530(LE_IFMODE eMode, LE_IFPOLARITY ePolarity, char fV35Select, LE_IFDRVCTRL eDrvCtrl);
  bool interfaceTTL(LE_IFLEVELEX eLevelEx, LE_IFPOLARITY ePolarity, LE_IFPOLARITY eClockPolarity, LE_IFOUTTYPE eOutType);
  bool interfaceLineControlOff();
  bool interfaceLineControlDTE(unsigned short usDelayRsOn, unsigned short usDelaySd, unsigned short usDelayRsOff, char fErSw);
  bool interfaceLineControlDCE(unsigned short usDelayCsOn, unsigned short usDelayCsOff, unsigned short usDelayCdOn, unsigned short usDelayRd, unsigned short usDelayCdOff);
  std::tuple<LE_IFPORT, LE_IFMODE, LE_IFPOLARITY, char, LE_IFDRVCTRL> getInterfaceMode();
  bool setIdleTimeRec(LE_IDLETIMEREC eIdleTimeRec);
  LE_IDLETIMEREC getIdleTimeRec();
  bool setTimeStampRec(LE_TIMESTAMPREC eTimeStampRec);
  LE_TIMESTAMPREC getTimeStampRec();
  bool setLineRec(char fLineStateRec);
  bool getLineRec();
  bool setCaptureBuf(LE_CAPTUREBUF eCaptureBuf);
  LE_CAPTUREBUF getCaptureBuf();
  LE_MONDATAFORMAT_VARIANT loadDT(std::string filename, int nFrames, int iFormat);
  bool isAbort();
  bool setBscTranslate(char fBscTranslate);
  char getBscTranslate();
  bool getScreen(std::string filename);
  unsigned long getState();
  int getBertDataCount();
  std::tuple<LE_DATABERT, int> getBertData();
  bool importPcap(std::string filename);
} // wrapper
#endif // WRAPPER_H